@@ -2,7 +2,7 @@ use core::convert::TryFrom;
 use core::fmt::Debug;
 use displaydoc::Display;
 use krakenrs::{
-    BsType, KrakenAPI, KrakenClientConfig, KrakenCredentials, LimitOrder, MarketOrder, OrderFlag,
+    BsType, KrakenCredentials, KrakenRestAPI, KrakenRestConfig, LimitOrder, MarketOrder, OrderFlag,
 };
 use serde::Serialize;
 use std::collections::{BTreeMap, BTreeSet};
@@ -39,6 +39,8 @@ enum Command {
     Ticker { pairs: Vec<String> },
     /// Get account balance
     GetBalance,
+    /// Get websockets token
+    GetWebSocketsToken,
     /// Get open orders list
     GetOpenOrders,
     /// Cancel order: {id}
@@ -80,27 +82,15 @@ fn log_value<T: Serialize + Debug>(val: &T) {
 fn main() {
     let config = KrakConfig::from_args();
 
-    let mut kc_config = KrakenClientConfig::default();
+    let mut kc_config = KrakenRestConfig::default();
 
     // Load credentials from disk if specified
     if let Some(creds) = config.creds {
-        let current_dir = std::env::current_dir().expect("Could not get current directory");
-        let path = current_dir.join(creds);
-        eprintln!("Credentials path: {:?}", path);
-        let creds_file =
-            std::fs::read_to_string(path).expect("Could not read specified credentials file");
-        let creds_data: KrakenCredentials =
-            serde_json::from_str(&creds_file).expect("Could not parse credentials file as json");
-        if creds_data.key.is_empty() {
-            panic!("Missing credentials 'key' value");
-        }
-        if creds_data.secret.is_empty() {
-            panic!("Missing credentials 'secret' value");
-        }
-        kc_config.creds = creds_data;
+        eprintln!("Credentials path: {:?}", creds);
+        kc_config.creds = KrakenCredentials::load_json_file(creds).expect("credential file error");
     }
 
-    let api = KrakenAPI::try_from(kc_config).expect("could not create kraken api");
+    let api = KrakenRestAPI::try_from(kc_config).expect("could not create kraken api");
 
     match config.command {
         Command::Time => {
@@ -131,6 +121,10 @@ fn main() {
             let sorted_result = result.into_iter().collect::<BTreeMap<_, _>>();
             log_value(&sorted_result);
         }
+        Command::GetWebSocketsToken => {
+            let result = api.get_websockets_token().expect("api call failed");
+            log_value(&result);
+        }
         Command::GetOpenOrders => {
             let result = api.get_open_orders(None).expect("api call failed");
             let sorted_result = result.open.into_iter().collect::<BTreeMap<_, _>>();
