@@ -1,9 +1,10 @@
-//! A rust crate for making requests to the Kraken API
+//! A rust crate for making requests to the Kraken Rest API and subscribing
+//! to Kraken websockets feeds
 
 #![deny(missing_docs)]
 
-mod kraken_client;
-pub use kraken_client::*;
+mod kraken_rest_client;
+pub use kraken_rest_client::*;
 
 mod messages;
 use messages::{
@@ -13,13 +14,17 @@ use messages::{
 pub use messages::{
     AddOrderResponse, AssetPairsResponse, AssetTickerInfo, AssetsResponse, BalanceResponse, BsType,
     CancelAllOrdersAfterResponse, CancelAllOrdersResponse, CancelOrderResponse,
-    GetOpenOrdersResponse, OrderAdded, OrderFlag, OrderInfo, OrderStatus, OrderType,
-    SystemStatusResponse, TickerResponse, TimeResponse, TxId, UserRefId,
+    GetOpenOrdersResponse, GetWebSocketsTokenResponse, OrderAdded, OrderFlag, OrderInfo,
+    OrderStatus, OrderType, SystemStatusResponse, TickerResponse, TimeResponse, TxId, UserRefId,
 };
 
 use core::convert::TryFrom;
 use std::collections::BTreeSet;
 
+// Websockets API support
+#[cfg(feature = "ws")]
+pub mod ws;
+
 /// A description of a market order to place
 #[derive(Debug, Clone)]
 pub struct MarketOrder {
@@ -48,30 +53,33 @@ pub struct LimitOrder {
     pub oflags: BTreeSet<OrderFlag>,
 }
 
-/// A connection to the Kraken API
+/// A connection to the Kraken REST API
 /// This only supports blocking http requests for now
-pub struct KrakenAPI {
-    client: KrakenClient,
+pub struct KrakenRestAPI {
+    client: KrakenRestClient,
 }
 
-impl KrakenAPI {
+impl KrakenRestAPI {
     /// (Public) Get the kraken system's time
     pub fn time(&self) -> Result<TimeResponse> {
         let result: Result<KrakenResult<TimeResponse>> = self.client.query_public("Time", Empty {});
         result.and_then(unpack_kraken_result)
     }
+
     /// (Public) Get the kraken system's status
     pub fn system_status(&self) -> Result<SystemStatusResponse> {
         let result: Result<KrakenResult<SystemStatusResponse>> =
             self.client.query_public("SystemStatus", Empty {});
         result.and_then(unpack_kraken_result)
     }
+
     /// (Public) Get the list of kraken's supported assets, and info
     pub fn assets(&self) -> Result<AssetsResponse> {
         let result: Result<KrakenResult<AssetsResponse>> =
             self.client.query_public("Assets", Empty {});
         result.and_then(unpack_kraken_result)
     }
+
     /// (Public) Get the list of kraken's asset pairs, and info
     ///
     /// Arguments:
@@ -85,6 +93,7 @@ impl KrakenAPI {
         );
         result.and_then(unpack_kraken_result)
     }
+
     /// (Public) Get the ticker price for one or more asset pairs
     ///
     /// Arguments:
@@ -98,12 +107,21 @@ impl KrakenAPI {
         );
         result.and_then(unpack_kraken_result)
     }
+
     /// (Private) Get the balance
     pub fn get_account_balance(&self) -> Result<BalanceResponse> {
         let result: Result<KrakenResult<BalanceResponse>> =
             self.client.query_private("Balance", Empty {});
         result.and_then(unpack_kraken_result)
     }
+
+    /// (Private) Get a websockets authentication token
+    pub fn get_websockets_token(&self) -> Result<GetWebSocketsTokenResponse> {
+        let result: Result<KrakenResult<GetWebSocketsTokenResponse>> =
+            self.client.query_private("GetWebSocketsToken", Empty {});
+        result.and_then(unpack_kraken_result)
+    }
+
     /// (Private) Get the list of open orders
     ///
     /// Arguments:
@@ -114,6 +132,7 @@ impl KrakenAPI {
             .query_private("OpenOrders", GetOpenOrdersRequest { userref });
         result.and_then(unpack_kraken_result)
     }
+
     /// (Private) Cancel order
     ///
     /// Arguments:
@@ -124,12 +143,14 @@ impl KrakenAPI {
             .query_private("CancelOrder", CancelOrderRequest { txid: id });
         result.and_then(unpack_kraken_result)
     }
+
     /// (Private) Cancel all orders (regardless of user ref or tx id)
     pub fn cancel_all_orders(&self) -> Result<CancelAllOrdersResponse> {
         let result: Result<KrakenResult<CancelAllOrdersResponse>> =
             self.client.query_private("CancelAll", Empty {});
         result.and_then(unpack_kraken_result)
     }
+
     /// (Private) Cancel all orders after
     ///
     /// Arguments:
@@ -197,11 +218,11 @@ impl KrakenAPI {
     }
 }
 
-impl TryFrom<KrakenClientConfig> for KrakenAPI {
+impl TryFrom<KrakenRestConfig> for KrakenRestAPI {
     type Error = Error;
-    fn try_from(src: KrakenClientConfig) -> Result<Self> {
-        Ok(KrakenAPI {
-            client: KrakenClient::try_from(src)?,
+    fn try_from(src: KrakenRestConfig) -> Result<Self> {
+        Ok(KrakenRestAPI {
+            client: KrakenRestClient::try_from(src)?,
         })
     }
 }
